@@ -1,6 +1,9 @@
 #| -*-Scheme-*-
 
-Copyright (C) 2006, 2007, 2008, 2009, 2010 Matthew Birkholz
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -30,7 +33,7 @@ USA.
 (define (c-generate library #!optional prefix)
   (let ((prefix (if (default-object? prefix) "" prefix))
 	(includes (include-cdecls library)))
-    (guarantee-string prefix 'c-generate)
+    (guarantee string? prefix 'c-generate)
     (let ((shim.c (string-append library "-shim.c")))
       (if c-generate-noisily?
 	  (with-notification
@@ -76,14 +79,14 @@ USA.
 (define (gen-callout-trampolines includes)
   (for-each
    (lambda (name.alienf)
-     (with-simple-restart 'CONTINUE "Continue generating callout trampolines."
+     (with-simple-restart 'continue "Continue generating callout trampolines."
        (lambda ()
 	 (bind-condition-handler
 	  (list condition-type:simple-error)
 	  (lambda (condition)
-	    (let ((restart (find-restart 'CONTINUE condition))
-		  (msg (access-condition condition 'MESSAGE))
-		  (irr (access-condition condition 'IRRITANTS)))
+	    (let ((restart (find-restart 'continue condition))
+		  (msg (access-condition condition 'message))
+		  (irr (access-condition condition 'irritants)))
 	      (apply warn msg irr)
 	      (if restart
 		  (invoke-restart restart))))
@@ -105,7 +108,7 @@ USA.
 	     (callout-restores name tos-var ret-var ret-ctype includes))
 	    (return
 	     (callout-return tos-var ret-var ret-ctype includes))
-	    (name (symbol-name name)))
+	    (name (symbol->string name)))
 	(write-string
 	 (string-append "
 SCM
@@ -123,10 +126,10 @@ Scm_continue_"name" (void)
 	  (inits (callout-inits ret-ctype params includes))
 	  (call (callout-call name ret-var ret-ctype params includes))
 	  (saves (callout-saves ret-var ret-ctype includes)))
-      (let ((name (symbol-name name)))
+      (let ((name (symbol->string name)))
 	(write-string
 	 (string-append "
-void
+SCM
 Scm_"name" (void)
 \{
   /* Declare. */" declares "
@@ -139,18 +142,17 @@ Scm_"name" (void)
   /* Save. */
   callout_unseal (&Scm_continue_"name");" saves "
 
-  callout_continue (&Scm_continue_"name");
-  /* NOTREACHED */
+  return callout_continue (&Scm_continue_"name");
 }
 "))))))
 
 (define (matching-param? string params)
-  (find-matching-item params
-    (lambda (param) (string=? string (symbol-name (car param))))))
+  (find (lambda (param) (string=? string (symbol->string (car param))))
+	params))
 
 (define (new-variable root-name params)
   ;; Returns a name (string) for a variable that must be distinct from
-  ;; those in the PARAMS alist.
+  ;; those in the params alist.
   (let loop ((n 0))
     (let ((name (string-append root-name (number->string n))))
       (if (not (matching-param? name params))
@@ -161,7 +163,7 @@ Scm_"name" (void)
 
 (define (callout-part2-decls tos-var ret-var ret-ctype includes)
   ;; Returns a multi-line string declaring the variables to be used in
-  ;; the second part of a callout trampoline.  See the Owner's Manual.
+  ;; the second part of a callout trampoline.  (See the FFI Manual.)
   (let ((ctype (definite-ctype ret-ctype includes))
 	(decl (decl-string ret-ctype)))
     (string-append "
@@ -172,7 +174,7 @@ Scm_"name" (void)
 
 (define (callout-restores name tos-var ret-var ret-ctype includes)
   (let* ((ctype (definite-ctype ret-ctype includes))
-	 (tramp2 (string-append "&Scm_continue_" (symbol-name name)))
+	 (tramp2 (string-append "&Scm_continue_" (symbol->string name)))
 	 (ret-decl (decl-string ret-ctype)))
     (string-append "
   "tos-var" = callout_lunseal ("tramp2");"
@@ -182,11 +184,21 @@ Scm_"name" (void)
 (define (callout-return tos-var ret-var ret-ctype includes)
   (let ((ctype (definite-ctype ret-ctype includes)))
     (string-append
-     (if (ctype/void? ctype)
-	 (string-append "
-  "ret-var"s = unspecific();")
-	 (string-append "
-  "ret-var"s = "(callout-return-converter ctype)" ("ret-var");")) "
+     (cond ((ctype/void? ctype)
+	    (string-append "
+  "ret-var"s = unspecific();"))
+	   ((or (ctype/struct? ctype) (ctype/union? ctype))
+	    (let ((decl (decl-string ret-ctype)))
+	      (string-append "
+  "ret-var"s = struct_to_scm (&"ret-var", sizeof("decl"));")))
+	   ((ctype/pointer? ctype)
+	    (string-append "
+  "ret-var"s = pointer_to_scm ("ret-var");"))
+	   ((or (ctype/basic? ctype) (ctype/enum? ctype))
+	    (let ((func (basic-scm-converter ctype)))
+	      (string-append "
+  "ret-var"s = "func" ("ret-var");")))
+	   (else (error "Unexpected callout return type:" ctype ret-ctype))) "
   callout_pop ("tos-var");
   return ("ret-var"s);")))
 
@@ -202,7 +214,7 @@ Scm_"name" (void)
 			    (if (null? params) '()
 				(cons
 				 (let* ((param (car params))
-					(name (symbol-name (car param)))
+					(name (symbol->string (car param)))
 					(type (cadr param))
 					(decl (decl-string type)))
 				   (string-append "
@@ -211,7 +223,10 @@ Scm_"name" (void)
 
 (define (callout-inits ret-ctype params includes)
   ;; Returns a multi-line string in C syntax for the Init section.
-  (let* ((alien-ret-arg? (ctype/pointer? (definite-ctype ret-ctype includes)))
+  (let* ((alien-ret-arg? (let ((definite (definite-ctype ret-ctype includes)))
+			   (or (ctype/pointer? definite)
+			       (ctype/struct? definite)
+			       (ctype/union? definite))))
 	 (nargs
 	  ;; (c-call 1:alien-function 2:ret-alien 3:arg1)
 	  ;; (c-call 1:alien-function 2:arg1)
@@ -227,7 +242,7 @@ Scm_"name" (void)
 			  (name (car param))
 			  (ctype (cadr param))
 			  (funcast (callout-arg-converter name ctype includes))
-			  (name (symbol-name name))
+			  (name (symbol->string name))
 			  (num (number->string n)))
 		     (string-append "
   "name" = "funcast" ("num");"))
@@ -241,9 +256,9 @@ Scm_"name" (void)
 
 (define (callout-call name ret-var ret-ctype params includes)
   ;; Returns a multi-line string in C syntax for the Call section.
-  (let ((name (symbol-name name))
+  (let ((name (symbol->string name))
 	(args (decorated-string-append
-	       "" ", " "" (map (lambda (param) (symbol-name (car param)))
+	       "" ", " "" (map (lambda (param) (symbol->string (car param)))
 			       params))))
     (if (not (ctype/void? (definite-ctype ret-ctype includes)))
 	(string-append "
@@ -253,7 +268,7 @@ Scm_"name" (void)
 
 (define (callout-arg-converter name arg-ctype includes)
   ;; Returns the name of the C function that takes an argument index
-  ;; and returns it as the C type ARG-CTYPE.  May have a cast
+  ;; and returns it as the C type arg-ctype.  May have a cast
   ;; expression at the beginning.  Handles args named CALLBACK and ID
   ;; specially.
   (let ((ctype (definite-ctype arg-ctype includes))
@@ -267,31 +282,30 @@ Scm_"name" (void)
 	  ((ctype/enum? ctype) "arg_long")
 	  ((ctype/basic? ctype)
 	   (case ctype
-	     ((CHAR SHORT INT LONG) "arg_long")
-	     ((UCHAR USHORT UINT ULONG) "arg_ulong")
-	     ((FLOAT DOUBLE) "arg_double")
+	     ((char short int long) "arg_long")
+	     ((uchar ushort uint ulong) "arg_ulong")
+	     ((float double) "arg_double")
 	     (else (error "Unexpected parameter type:" arg-ctype))))
+	  ((or (ctype/struct? ctype) (ctype/union? ctype))
+	   (string-append "*("decl"*) arg_pointer"))
 	  (else (error "Unexpected parameter type:" arg-ctype)))))
 
-(define (callout-return-converter ctype)
+(define (basic-scm-converter ctype)
   ;; Returns the name of a C function that converts from the definite
-  ;; C type CTYPE to the analogous Scheme object.  Note that the
-  ;; pointer converter, pointer_to_scm, returns pointers via c-call's
-  ;; second argument.
-  (cond ((ctype/pointer? ctype) "pointer_to_scm")
-	((ctype/enum? ctype) "ulong_to_scm")
+  ;; C type ctype to the analogous Scheme object.
+  (cond ((ctype/enum? ctype) "ulong_to_scm")
 	((ctype/basic? ctype)
 	 (case ctype
-	   ((CHAR SHORT INT LONG) "long_to_scm")
-	   ((UCHAR USHORT UINT ULONG) "ulong_to_scm")
-	   ((FLOAT DOUBLE) "double_to_scm")
-	   ((VOID) #f)
-	   (else (error "Unexpected return type:" ctype))))
-	(else (error "Unexpected return type:" ctype))))
+	   ((char short int long) "long_to_scm")
+	   ((uchar ushort uint ulong) "ulong_to_scm")
+	   ((float double) "double_to_scm")
+	   ((void) #f)
+	   (else (error "Unexpected C type:" ctype))))
+	(else (error "Unexpected C type:" ctype))))
 
 (define (callout-return-variable params)
   ;; Returns a name (string) for a variable that will hold the return
-  ;; value.  Checks for two name collisions with the PARAMS, e.g. ret0
+  ;; value.  Checks for two name collisions with the params, e.g. ret0
   ;; and ret0s, the latter being the SCM version of the return value.
   (let loop ((n 0))
     (let* ((ns (number->string n))
@@ -305,26 +319,26 @@ Scm_"name" (void)
 	      (loop (1+ n)))))))
 
 (define (decl-string ctype)
-  ;; Returns a string in C syntax declaring the C type CTYPE.
+  ;; Returns a string in C syntax declaring the C type ctype.
   ;; E.g. given (* |GtkWidget|), returns "GtkWidget *".
   (cond	((eq? ctype '*) "void*")
 	((eq? ctype 'uchar) "unsigned char")
 	((eq? ctype 'ushort) "unsigned short")
 	((eq? ctype 'uint) "unsigned int")
 	((eq? ctype 'ulong) "unsigned long")
-	((symbol? ctype) (symbol-name ctype))
+	((symbol? ctype) (symbol->string ctype))
 	((ctype/pointer? ctype)
 	 (string-append (decl-string (ctype-pointer/target-type ctype))
 			" *"))
 	((ctype/const? ctype)
-	 (string-append "const "
-			(decl-string (ctype-const/qualified-type ctype))))
+	 (string-append (decl-string (ctype-const/qualified-type ctype))
+			" const"))
 	((ctype/struct-name? ctype)
-	 (string-append "struct " (symbol-name (ctype-struct/name ctype))))
+	 (string-append "struct " (symbol->string (ctype-struct/name ctype))))
 	((ctype/union-name? ctype)
-	 (string-append "union " (symbol-name (ctype-union/name ctype))))
+	 (string-append "union " (symbol->string (ctype-union/name ctype))))
 	((ctype/enum-name? ctype)
-	 (string-append "enum " (symbol-name (ctype-enum/name ctype))))
+	 (string-append "enum " (symbol->string (ctype-enum/name ctype))))
 	(else
 	 (error "Could not generate a C type declaration:" ctype))))
 
@@ -334,14 +348,14 @@ Scm_"name" (void)
 (define (gen-callback-trampolines includes)
   (for-each
    (lambda (name.alienf)
-     (with-simple-restart 'CONTINUE "Continue generating callback trampolines."
+     (with-simple-restart 'continue "Continue generating callback trampolines."
        (lambda ()
 	 (bind-condition-handler
 	  (list condition-type:simple-error)
 	  (lambda (condition)
-	    (let ((restart (find-restart 'CONTINUE condition))
-		  (msg (access-condition condition 'MESSAGE))
-		  (irr (access-condition condition 'IRRITANTS)))
+	    (let ((restart (find-restart 'continue condition))
+		  (msg (access-condition condition 'message))
+		  (irr (access-condition condition 'irritants)))
 	      (apply warn msg irr)
 	      (if restart
 		  (invoke-restart restart))))
@@ -360,7 +374,7 @@ Scm_"name" (void)
       (let ((declares (callback-decls params))
 	    (restores (callback-restores params tos-var))
 	    (constructs (callback-conses params args-var includes))
-	    (name (symbol-name name)))
+	    (name (symbol->string name)))
 	(write-string
 	 (string-append "
 static void
@@ -385,7 +399,7 @@ Scm_kernel_"name" (void)
 	  (saves (callback-saves params))
 	  (return (callback-return ret-ctype includes))
 	  (ret-decl (decl-string ret-ctype))
-	  (name (symbol-name name)))
+	  (name (symbol->string name)))
       (write-string
        (string-append
 	"
@@ -401,7 +415,7 @@ Scm_"name" ("arglist")
   ;; the second (inner, kernel) part of a callback trampoline.
   (apply string-append (map (lambda (param)
 			      (let ((decl (decl-string (cadr param)))
-				    (name (symbol-name (car param))))
+				    (name (symbol->string (car param))))
 				(string-append "
   "decl" "name";")))
 			    params)))
@@ -409,7 +423,7 @@ Scm_"name" ("arglist")
 (define (callback-restores params tos-var)
   ;; Returns a multi-line string setting the params from the C data stack.
   (apply string-append (map (lambda (param)
-			      (let ((name (symbol-name (car param)))
+			      (let ((name (symbol->string (car param)))
 				    (decl (decl-string (cadr param))))
 				(string-append "
   CSTACK_LPOP ("decl", "name", "tos-var");")))
@@ -423,7 +437,7 @@ Scm_"name" ("arglist")
 		      (ctype (cadr param)))
 		  (if (eq? name '|ID|)
 		      ""
-		      (let ((name (symbol-name name)))
+		      (let ((name (symbol->string name)))
 			(let ((construction
 			       (callback-arg-cons name ctype includes)))
 			  (string-append "
@@ -435,13 +449,13 @@ Scm_"name" ("arglist")
    "" ", " ""				;prefix, infix, suffix
    (map (lambda (param)
 	  (string-append (decl-string (cadr param))
-			 " " (symbol-name (car param))))
+			 " " (symbol->string (car param))))
 	params)))
 
 (define (callback-saves params)
   (apply string-append
    (map (lambda (param)
-	  (let ((name (symbol-name (car param)))
+	  (let ((name (symbol->string (car param)))
 		(ctype (cadr param)))
 	    (string-append "
   CSTACK_PUSH ("(decl-string ctype)", "name");")))
@@ -449,7 +463,7 @@ Scm_"name" ("arglist")
 
 (define (callback-return ret-type includes)
   ;; Returns a multi-line string that returns from a callback
-  ;; trampoline with a value of type RET-TYPE, converted from
+  ;; trampoline with a value of type ret-type, converted from
   ;; val_register.
   (let ((funcast (callback-return-converter ret-type includes)))
     (if (not funcast) "
@@ -459,16 +473,21 @@ Scm_"name" ("arglist")
 
 (define (callback-arg-cons arg-name arg-ctype includes)
   ;; Returns a function call that applies the appropriate Scheme
-  ;; constructor to the ARG-CTYPE variable ARG-NAME.
+  ;; constructor to the arg-ctype variable arg-name.
   (let ((ctype (definite-ctype arg-ctype includes)))
-    (if (ctype/pointer? ctype)
-	(string-append "cons_alien((void*)"arg-name")")
-	(let ((func (callout-return-converter ctype)))
-	  (string-append func"("arg-name")")))))
+    (cond ((ctype/pointer? ctype)
+	   (string-append "cons_alien((void*)"arg-name")"))
+	  ((or (ctype/struct? ctype) (ctype/union? ctype))
+	   (error "Unsupported callback argument type:" arg-ctype arg-name))
+	  ((or (ctype/basic? ctype) (ctype/enum? ctype))
+	   (let ((func (basic-scm-converter ctype)))
+	     (string-append func"("arg-name")")))
+	  (else
+	   (error "Unexpected callback argument type:" arg-ctype arg-name)))))
 
 (define (callback-return-converter ret-type includes)
   ;; Returns the name of the C function that takes no arguments and
-  ;; returns the interpreter's VAL register as the C type RET-CTYPE.
+  ;; returns the interpreter's Val register as the C type ret-ctype.
   (let ((ctype (definite-ctype ret-type includes)))
     (cond ((ctype/pointer? ctype)
 	   (string-append "("(decl-string ret-type)")pointer_value"))
@@ -476,11 +495,11 @@ Scm_"name" ("arglist")
 	  ((ctype/void? ctype) #f)
 	  ((ctype/basic? ctype)
 	   (case ctype
-	     ((CHAR SHORT INT LONG) "long_value")
-	     ((UCHAR USHORT UINT ULONG) "ulong_value")
-	     ((FLOAT DOUBLE) "double_value")
-	     (else (error "Unexpected return type:" ctype))))
-	  (else (error "Unexpected return type:" ctype)))))
+	     ((char short int long) "long_value")
+	     ((uchar ushort uint ulong) "ulong_value")
+	     ((float double) "double_value")
+	     (else (error "Unexpected callback return type:" ctype))))
+	  (else (error "Unexpected callback return type:" ctype)))))
 
 
 ;;; Groveler
@@ -498,7 +517,7 @@ Scm_"name" ("arglist")
 
 #include <stdio.h>
 " (basics-grovel-func) (enums-grovel-func includes)))
-      (flush-output)
+      (flush-output-port)
       (let* ((structs (gen-struct-grovel-funcs includes))
 	     (unions (gen-union-grovel-funcs includes)))
 	(let ((library (c-includes/library includes)))
@@ -542,7 +561,7 @@ grovel_basics (FILE * out)
 	  (map (lambda (entry)
 		 (let* ((name (car entry))
 			(decl (decl-string name))
-			(name (symbol-name name)))
+			(name (symbol->string name)))
 		   (string-append "
   fprintf (out, \"   ((sizeof "name") . %ld)\\n\", (long) sizeof ("decl"));")))
 		    peek-poke-primitives))
@@ -558,7 +577,7 @@ grovel_enums (FILE * out)
 \{"
    (apply string-append
 	  (map (lambda (constant)
-		 (let ((name (symbol-name (car constant))))
+		 (let ((name (symbol->string (car constant))))
 		   (string-append "
   fprintf (out, \"   (|"name"| . %ld)\\n\", ((long)"name"));")))
 	       (c-includes/enum-constants includes)))
@@ -568,53 +587,55 @@ grovel_enums (FILE * out)
 
 (define (gen-struct-grovel-funcs includes)
   ;; Returns the names of the generated functions.
-  (append-map*!
-   (map (lambda (name.info)
-	  ;; The named structs, top-level OR internal.
-	  (let ((name (list 'STRUCT (car name.info))))
-	    (gen-struct-union-grovel-func name includes)))
-	(c-includes/structs includes))
-   (lambda (name.info)
+  (fold-right
+   (lambda (name.info result)
      ;; Typedefs giving names to struct types.
      (let* ((name (car name.info))
 	    (ctype (definite-ctype name includes)))
        (if (ctype/struct? ctype)
-	   (list (gen-struct-union-grovel-func name includes))
-	   '())))
+	   (cons (gen-struct-union-grovel-func name includes)
+		 result)
+	   result)))
+   (map (lambda (name.info)
+	  ;; The named structs, top-level OR internal.
+	  (let ((name (list 'struct (car name.info))))
+	    (gen-struct-union-grovel-func name includes)))
+	(c-includes/structs includes))
    (c-includes/type-names includes)))
 
 (define (gen-union-grovel-funcs includes)
   ;; Returns the names of the generated functions.
-  (append-map*!
-   (map (lambda (name.info)
-	  ;; The named unions, top-level OR internal.
-	  (let ((name (list 'UNION (car name.info))))
-	    (gen-struct-union-grovel-func name includes)))
-	(c-includes/unions includes))
-   (lambda (name.info)
+  (fold-right
+   (lambda (name.info result)
      ;; Typedefs giving names to union types.
      (let* ((name (car name.info))
 	    (ctype (definite-ctype name includes)))
        (if (ctype/union? ctype)
-	   (list (gen-struct-union-grovel-func name includes))
-	   '())))
+	   (cons (gen-struct-union-grovel-func name includes)
+		 result)
+	   result)))
+   (map (lambda (name.info)
+	  ;; The named unions, top-level OR internal.
+	  (let ((name (list 'union (car name.info))))
+	    (gen-struct-union-grovel-func name includes)))
+	(c-includes/unions includes))
    (c-includes/type-names includes)))
 
 (define (gen-struct-union-grovel-func name includes)
-  ;; Generate C code for a grovel_NAME function.
+  ;; Generate C code for a grovel_<name> function.
   (let ((fname (cond ((ctype/struct-name? name)
 		      (string-append "grovel_struct_"
-				     (symbol-name (ctype-struct/name name))))
+				     (symbol->string (ctype-struct/name name))))
 		     ((ctype/union-name? name)
 		      (string-append "grovel_union_"
-				     (symbol-name (ctype-union/name name))))
+				     (symbol->string (ctype-union/name name))))
 		     ((symbol? name)
-		      (string-append "grovel_type_" (symbol-name name)))
+		      (string-append "grovel_type_" (symbol->string name)))
 		     (else (error "Unexpected name:" name))))
 	(ctype (definite-ctype name includes))
 	(decl (decl-string name))
 	(_ (lambda args (for-each write-string args))))
-    (let ((key (list 'SIZEOF name)))
+    (let ((key (list 'sizeof name)))
       (_ "
 void
 "fname" (FILE * out)
@@ -625,8 +646,8 @@ void
      ctype includes
      (lambda (path brief-type)
        (let ((path (decorated-string-append
-		    "" "." "" (map symbol-name path)))
-	     (key (cons* 'OFFSET name path)))
+		    "" "." "" (map symbol->string path)))
+	     (key (cons* 'offset name path)))
 	 (_ "
   fprintf (out, \"   (")(write key)(_" %ld . ")(write brief-type)(_")\\n\", (long)((char*)&(S."path") - (char*)&S));"))))
     (_ "
@@ -635,12 +656,12 @@ void
     fname))
 
 (define (for-each-member-path ctype includes receiver)
-  ;; Calls RECEIVER with a path and an abbreviated type for each
-  ;; member (and nested member) of the struct or union CTYPE (a C
-  ;; struct or union type).  Each path is a list of member names
-  ;; (symbols) -- one name for immediate members, multiple names for
-  ;; nested members.  An abbreviated type is a Ctype, but is 'ENUM if
-  ;; the actual type is (ENUM ...).
+  ;; Calls receiver with a path and an abbreviated type for each
+  ;; member (and nested member) of ctype, a C struct or union type.
+  ;; Each path is a list of member names (symbols) -- one name for
+  ;; immediate members, multiple names for nested members.  An
+  ;; abbreviated type is a C type, but is 'enum if the actual type is
+  ;; (enum ...).
 
   (let ((type (ctype-definition ctype includes)))
     (cond ((ctype/struct-defn? type)
@@ -669,7 +690,7 @@ void
 		 (ctype/array? ctype))
 	     (receiver (list name) type))
 	    ((ctype/enum? ctype)
-	     (receiver (list name) 'ENUM))
+	     (receiver (list name) 'enum))
 	    ((ctype/struct-defn? ctype)
 	     (receiver (list name) type)
 	     (let ((new-stack (cons type stack)))
