@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -34,8 +34,8 @@ extern unsigned short x87_read_control_word (void);
 extern void x87_write_control_word (unsigned short);
 extern unsigned short x87_read_status_word (void);
 /* extern void x87_write_status_word (unsigned short);	No f(n)stsw. */
-extern void x87_read_environment (unsigned char *);
-extern void x87_write_environment (const unsigned char *);
+extern void x87_read_environment (struct x87_fenv *);
+extern void x87_write_environment (const struct x87_fenv *);
 
 #ifndef x87_p
 #  ifdef HAVE_X87
